#include "Texture.h"
#include "Vector2.h"
#include <fstream>
#include "Pixel.h"

Texture::Texture()
{

}

Texture::Texture(int sizeX, int sizeY) : sizeX(sizeX), sizeY(sizeY)
{
	array = new Pixel[sizeX * sizeY];
}
Texture::Texture(char * path_file, int sizeX, int sizeY, bool font) : sizeX(sizeX), sizeY(sizeY)
{
	array = new Pixel[sizeX * sizeY];
	load_from_file(path_file);

	if(font)
		for(int i = 0; i < sizeX * sizeY; i++)
			array[i] = Pixel(' ', 0xFF, array[i].alpha);
}

void Texture::load_from_file(char * path_file)
{
	std::ifstream sign_file((std::string)path_file + ".txt");
	std::ifstream alpha_file((std::string)path_file + ".alpha");
	std::ifstream color_file((std::string)path_file + ".color");

	int s_texture = sizeX * sizeY;

	for(int i = 0; i < s_texture; i++)
	{
		array[i].sign = sign_file.get();
		array[i].alpha = alpha_file.get();
		array[i].color = color_file.get();
	}
}

void Texture::copy_texture(Vector2<unsigned int> position, Texture * texture, int rotate)
{
	if(texture == nullptr)
		return;
	if(rotate == 0)
		for(int j = 0; j < texture->sizeY; j++)
		{
			for(int i = 0; i < texture->sizeX; i++)
			{
				int index_t = i + j * texture->sizeX, index_r = position.x + i + ( position.y + j ) * sizeX;
				array[index_r].sign = texture->array[index_t].sign;
				array[index_r].color = texture->array[index_t].color;
				array[index_r].alpha = texture->array[index_t].alpha;
			}
		}
	else if(rotate == 1)
		for(int j = 0, a = texture->sizeY; j < texture->sizeY; j++, --a)
		{
			for(int i = 0, b = texture->sizeX; i < texture->sizeX; i++, --b)
			{
				int index_t = b * texture->sizeX - a, index_r = position.x + i + ( position.y + j ) * sizeX;
				array[index_r].sign = texture->array[index_t].sign;
				array[index_r].color = texture->array[index_t].color;
				array[index_r].alpha = texture->array[index_t].alpha;
			}
		}
	else if(rotate == 2)
		for(int j = 0, a = texture->sizeY; j < texture->sizeY; j++, --a)
		{
			for(int i = 0, b = texture->sizeX; i < texture->sizeX; i++, --b)
			{
				int index_t = a * texture->sizeX - b, index_r = position.x + i + ( position.y + j ) * sizeX;
				array[index_r].sign = texture->array[index_t].sign;
				array[index_r].color = texture->array[index_t].color;
				array[index_r].alpha = texture->array[index_t].alpha;
			}
		}
	else if(rotate == 3)
		for(int j = 0; j < texture->sizeY; j++)
		{
			for(int i = 0, b = texture->sizeX; i < texture->sizeX; i++, --b)
			{
				int index_t = b * texture->sizeX - j - 1, index_r = position.x + i + ( position.y + j ) * sizeX;
				array[index_r].sign = texture->array[index_t].sign;
				array[index_r].color = texture->array[index_t].color;
				array[index_r].alpha = texture->array[index_t].alpha;
			}
		}
}

