#pragma once

template <typename type>
struct Vector2
{
	type x;
	type y;

	Vector2() : x(0), y(0) {}

	Vector2(type x, type y) : x(x), y(y) {}

	template <typename to_type>
	Vector2(Vector2<to_type> vector) : x(static_cast<to_type>( vector.x )), y(static_cast<to_type>( vector.y )) {}
};


/*##########*//*##########*//*##########*/

typedef Vector2<int> Vector2i;
typedef Vector2<float> Vector2f;
typedef Vector2<unsigned int> Vector2u;


/*##########*//*##########*//*##########*/
template <typename type>
inline Vector2<type> operator+(Vector2<type> left, Vector2<type> right)
{
	return Vector2<type>{ left.x + right.x, left.y + right.y };
}
template <typename type>
inline Vector2<type> operator+(Vector2<type> left, type right)
{
	return Vector2<type>{ left.x + right, left.y + right };
}

/*##########*//*##########*//*##########*/
template <typename type>
inline Vector2<type> operator-(Vector2<type> left, Vector2<type> right)
{
	return Vector2<type>{ left.x - right.x, left.y - right.y };
}
template <typename type>
inline Vector2<type> operator-(Vector2<type> left, type right)
{
	return Vector2<type>{ left.x - right, left.y - right };
}

/*##########*//*##########*//*##########*/
template <typename type>
inline Vector2<type> operator*(Vector2<type> left, type right)
{
	return Vector2<type>{ left.x * right, left.y * right };
}

template <typename type>
inline Vector2<type> operator*(type left, Vector2<type>  right)
{
	return Vector2<type>{ right.x * left, right.y * left };
}

template <typename type>
inline Vector2<type> operator*(Vector2<type> left, Vector2<type>  right)
{
	return Vector2<type>{ right.x * left.x, right.y * left.y };
}

/*##########*//*##########*//*##########*/
template <typename type>
inline Vector2<type> operator/(Vector2<type> left, type right)
{
	return Vector2<type>{ left.x / right, left.y / right };
}

template <typename type>
inline Vector2<type> operator/(type left, Vector2<type>  right)
{
	return Vector2<type>{ right.x / left, right.y / left };
}

template <typename type>
inline Vector2<type> operator/(Vector2<type> left, Vector2<type>  right)
{
	return Vector2<type>{ right.x / left.x, right.y / left.y };
}

/*##########*//*##########*//*##########*/
template <typename type>
inline bool operator==(Vector2<type> left, Vector2<type>  right)
{
	if((left.x == right.x) && ( left.y == right.y))
		return true;
	return false;
}

template <typename type>
inline bool operator!=(Vector2<type> left, Vector2<type>  right)
{
	if(( left.x == right.x ) && ( left.y == right.y ))
		return false;
	return true;
}

template <typename type>
inline bool operator<=>(Vector2<type> left, Vector2<type>  right)
{
	if(( left.x >= right.x - 1 && left.x <= right.x + 1 ) && ( left.y >= right.y - 1 && left.y <= right.y + 1 ))
		return true;
	return false;
}
