#include "Window.h"
#include "Render.h"
#include "Vector2.h"
#include <iostream>
#include "Pixel.h"
#include <chrono>
#include "Texture.h"
#include "World.h"
#include "Block.h"
#include "Pacman.h"
#include <conio.h>
#include <thread>
#include "Ghost.h"

enum Commands { UP = 72, DOWN = 80, LEFT = 75, RIGHT = 77 };

inline bool collisian_test(Vector2f position, World * world, Vector2f velocity)
{
	if(velocity.y > 0)
	{
		Vector2u pos1(( position + Vector2f(0.f, 5.f) ) / 5.f), pos2(( position + Vector2f(4.f, 5.f) ) / 5.f);

		if(!( ( world->get(pos1).can_be ) && ( world->get(pos2).can_be ) ))
		{
			return false;
		}
	}
	if(velocity.y < 0)
	{
		Vector2u pos1(( position + Vector2f(0.f, -1.f) ) / 5.f), pos2(( position + Vector2f(4.f, -1.f) ) / 5.f);

		if(!( ( world->get(pos1).can_be ) && ( world->get(pos2).can_be ) ))
		{
			return false;
		}
	}
	if(velocity.x > 0)
	{
		Vector2u pos1(( position + Vector2f(5.f, 0.f) ) / 5.f), pos2(( position + Vector2f(5.f, 4.f) ) / 5.f);

		if(!( ( world->get(pos1).can_be ) && ( world->get(pos2).can_be ) ))
		{
			return false;
		}
	}
	if(velocity.x < 0)
	{
		Vector2u pos1(( position + Vector2f(-1.f, 0.f) ) / 5.f), pos2(( position + Vector2f(-1.f, 4.f) ) / 5.f);

		if(!( ( world->get(pos1).can_be ) && ( world->get(pos2).can_be ) ))
		{
			return false;
		}
	}

	return true;
}

inline void stop_velocity(Vector2f * position, World * world, Vector2f * velocity)
{
	if(!collisian_test(*position, world, *velocity))
		*velocity = { 0, 0 };
}

inline void control(Pacman * pacman, World * world, int * command)
{
	float speed = 20;

	switch(*command)
	{
	case UP:
		if(collisian_test(pacman->position, world, Vector2f(0, -1 * speed)))
			pacman->velocity = { 0, -1 * speed }; break;
	case DOWN:
		if(collisian_test(pacman->position, world, Vector2f(0, 1 * speed)))
			pacman->velocity = { 0, 1 * speed }; break;
	case LEFT:
		if(collisian_test(pacman->position, world, Vector2f(-1 * speed, 0)))
			pacman->velocity = { -1 * speed, 0 }; break;
	case RIGHT:
		if(collisian_test(pacman->position, world, Vector2f(1 * speed, 0)))
			pacman->velocity = { 1 * speed, 0 }; break;
	}
}

inline Vector2f rand_velocity(Ghost * ghost, World * world)
{
	srand(std::chrono::steady_clock::now().time_since_epoch().count());
	int rotate = rand() % 4;
	float speed = 20;

	while(true)
	{
		switch(rotate)
		{
		case 0:
			if(collisian_test(ghost->position, world, Vector2f(0, -1 * speed)))
			{
				if(Vector2f( 0, -1 * speed ) != ghost->velocity * -1.f )
					return { 0, -1 * speed };
			} break;
		case 1:
			if(collisian_test(ghost->position, world, Vector2f(0, 1 * speed)))
			{
				if(Vector2f(0, 1 * speed) != ghost->velocity * -1.f)
					return { 0, 1 * speed };
			} break;
		case 2:
			if(collisian_test(ghost->position, world, Vector2f(-1 * speed, 0)))
			{
				if(Vector2f(-1 * speed, 0) != ghost->velocity * -1.f)
					return { -1 * speed, 0 };
			} break;
		case 3:
			if(collisian_test(ghost->position, world, Vector2f(1 * speed, 0)))
			{
				if(Vector2f(1 * speed, 0) != ghost->velocity * -1.f)
					return { 1 * speed, 0 };
			} break;
		}
		rotate = rand() % 4;
	}
}

void tick(Pacman * pacman, Ghost * ghost, World * world, int * command, std::vector<Block> * list, Texture * world_t, Texture * air_t, int * score)
{
	int tps = 0, ltps = 0;

	std::vector<Vector2u> vertex = { Vector2u(1,1), Vector2u(6, 1),Vector2u(12,1), Vector2u(15,1), Vector2u(21, 1), Vector2u(26,1),
	Vector2u(1, 5), Vector2u(6, 5), Vector2u(9, 5),Vector2u(12, 5), Vector2u(15, 5), Vector2u(18, 5), Vector2u(21, 5), Vector2u(26, 5),
	Vector2u(1, 8), Vector2u(6, 8), Vector2u(9, 8), Vector2u(12, 8), Vector2u(15, 8), Vector2u(18, 8), Vector2u(21, 8),Vector2u(26, 8),
	Vector2u(6,14), Vector2u(9, 14), Vector2u(18, 14), Vector2u(21, 14),
	Vector2u(9, 17), Vector2u(18, 17),
	Vector2u(1, 20), Vector2u(6, 20), Vector2u(12, 20), Vector2u(15, 20), Vector2u(18, 20), Vector2u(21, 20), Vector2u(26, 20),
	Vector2u(1, 23), Vector2u(3, 23), Vector2u(6, 23), Vector2u(9, 23), Vector2u(12, 23), Vector2u(15, 23), Vector2u(21, 23), Vector2u(24, 23),Vector2u(26, 23),
	Vector2u(1, 26), Vector2u(3, 26), Vector2u(6, 26), Vector2u(9, 26), Vector2u(12, 26), Vector2u(15, 26), Vector2u(21, 26), Vector2u(24, 26),Vector2u(26, 26),
	Vector2u(1, 29), Vector2u(12, 29), Vector2u(15, 29), Vector2u(26,29) };

	auto t_time = std::chrono::steady_clock::now();
	while(true)
	{
		auto start_move_clock = std::chrono::steady_clock::now();
		if(t_time + std::chrono::seconds(1) < std::chrono::steady_clock::now())
		{
			tps = ltps;  ltps = 0;
			t_time = std::chrono::steady_clock::now();
		}

		if(ghost->position <=> pacman->position)
		{
			return;
		}

		if(ghost->position <=> Vector2f(13 * 5, 13 * 5 ))
		{
			ghost->velocity = rand_velocity(ghost, world);
		}

		for(int i = 0; i < vertex.size(); i++)
		{
			if(ghost->position <=> (Vector2f)vertex[i] * 5.f)
			{
				ghost->velocity = rand_velocity(ghost, world);
			}
		}

		if(!collisian_test(ghost->position, world, ghost->velocity))
		{
			ghost->velocity = rand_velocity(ghost, world);
		}

		if(pacman->position.x > 0 && pacman->position.x < 1 && pacman->velocity.x < 0)
		{
			pacman->position.x = 28 * 5;
		}
		if(pacman->position.x > float(28 * 5 - 5) && pacman->position.x < float(28 * 5 - 1) && pacman->velocity.x > 0)
		{
			pacman->position.x = 1;
		}

		if(ghost->position.x > 0 && ghost->position.x < 1 && ghost->velocity.x < 0)
		{
			ghost->position.x = 28 * 5;
		}
		if(ghost->position.x > float(28 * 5 - 6) && ghost->position.x < float(28 * 5) && ghost->velocity.x > 0)
		{
			ghost->position.x = 1;
		}

		if(world->get((pacman->position) / 5.f).id == list->operator[](7).id)
		{
			world->get((pacman->position) / 5.f) = list->operator[](0);
			*score = *score + 1;
			world_t->copy_texture(Vector2u(pacman->position / 5.f) * 5U, air_t, 0 );
		}

		stop_velocity(&pacman->position, world, &pacman->velocity);
		if(_kbhit())
		{
			*command = _getch();
		}
		control(pacman, world, command);

		if(*score == 288)
			return;

		ltps++;

		auto end_move_clock = std::chrono::steady_clock::now();
		pacman->update_pos(std::chrono::duration<float>(end_move_clock - start_move_clock).count());
		ghost->update_pos(std::chrono::duration<float>(end_move_clock - start_move_clock).count());

	}
}

void frame(Texture * pacman_t, Texture * ghost_t, Texture * world_t, Render * render, Window * window, Pacman * pacman, Ghost * ghost, Texture * number_t, int * score)
{
	int fps = 0, lfps = 0;

	render->clear();

	auto f_time = std::chrono::steady_clock::now();
	while(true)
	{
		if(f_time + std::chrono::seconds(1) < std::chrono::steady_clock::now())
		{
			fps = lfps;  lfps = 0;
			f_time = std::chrono::steady_clock::now();
		}

		render->copy_texture({ 0,0 }, world_t, 0);
		render->copy_texture(Vector2u(pacman->position - 1.f), pacman_t, 0);
		render->copy_texture(Vector2u(ghost->position - 1.f), ghost_t, 0);

		std::string sc_str = std::to_string(*score); int size = sc_str.size();
		for(int i = 0; i < size; i++)
		{
			int index = sc_str[i] - '0';
			render->copy_texture(Vector2u(i * 5, 32 * 5), &number_t[index], 0);
		}

		render->update(window);
		render->clear();

		lfps++;
	}
}

int main()
{
	int command, score = 0;
	Pacman * pacman = new Pacman({ 13.55 * 5,17 * 5 }, { 0,0 });

	Ghost * ghost = new Ghost({ 13.55 * 5,14 * 5 }, { 0, -1 * 15 });

	Window * window = new Window({ 28 * 5, 36 * 5 });
	Render * render = new Render(*window->size);

	Texture * air_t = new Texture(5,5);
	for(int i = 0; i < 25; i++)
		*air_t->array = Pixel(' ', 0, true);

	Texture * point_t = new Texture((char *)"asserts\\textures\\blocks\\point", 5, 5);
	Texture * pacman_t = new Texture((char *)"asserts\\textures\\entities\\pacman", 7, 7);
	Texture * corner_t = new Texture((char *)"asserts\\textures\\blocks\\corner", 5, 5);
	Texture * wall_t = new Texture((char *)"asserts\\textures\\blocks\\wall", 5, 5);
	Texture * ghost_t = new Texture((char *)"asserts\\textures\\entities\\ghost", 7, 7);

	Texture * number_t = new Texture[10];
	for(int i = 0; i < 10; i++)
	{
		std::string path = "asserts\\textures\\font\\" + std::string(std::to_string(i));
		number_t[i] = Texture((char*)path.c_str(), 4, 5);
	}

	Texture * pixel_t = new Texture(1, 1);
	pixel_t->array[0] = Pixel('@', 0xAA, true);

	std::vector<Block> list = { Block(air_t, true, 0, 0), Block(wall_t, false, 0, 1), Block(wall_t, false, 1, 2),
		Block(corner_t, false, 0, 3),Block(corner_t, false, 1, 4),Block(corner_t, false, 2, 5),Block(corner_t, false, 3, 6), Block(point_t, true, 0, 7) };

	World * world = new World({ 28,31 }, list, (char *)"asserts\\world");

	Texture * world_t = new Texture(28 * 5, 31 * 5);
	for(unsigned int x = 0; x < 28; x++)
		for(unsigned int y = 0; y < 31; y++)
		{
			world_t->copy_texture({ x * 5, y * 5 }, world->array[x + y * 28].texture, world->array[x + y * 28].rotate);
		}

	std::thread t(tick, pacman, ghost, world, &command, &list, world_t, air_t, &score);
	std::thread f(frame, pacman_t, ghost_t, world_t, render, window, pacman, ghost, number_t, &score);

	t.join();
	f.detach();

	render->clear();

	while(true);
}